from flask import json
from authlib.common.security import generate_token
from authlib.common.urls import urlparse, url_decode
from authlib.oauth2.rfc6749 import grants
from authlib.oauth2.rfc7636 import (
    CodeChallenge as _CodeChallenge,
    create_s256_code_challenge,
)
from .models import db, User, Client
from .models import CodeGrantMixin, save_authorization_code
from .oauth2_server import TestCase
from .oauth2_server import create_authorization_server


class AuthorizationCodeGrant(CodeGrantMixin, grants.AuthorizationCodeGrant):
    TOKEN_ENDPOINT_AUTH_METHODS = ['client_secret_basic', 'client_secret_post', 'none']

    def save_authorization_code(self, code, request):
        return save_authorization_code(code, request)


class CodeChallenge(_CodeChallenge):
    SUPPORTED_CODE_CHALLENGE_METHOD = ['plain', 'S256', 'S128']


class CodeChallengeTest(TestCase):
    def prepare_data(self, token_endpoint_auth_method='none'):
        server = create_authorization_server(self.app)
        server.register_grant(
            AuthorizationCodeGrant,
            [CodeChallenge(required=True)]
        )

        user = User(username='foo')
        db.session.add(user)
        db.session.commit()

        client_secret = ''
        if token_endpoint_auth_method != 'none':
            client_secret = 'code-secret'

        client = Client(
            user_id=user.id,
            client_id='code-client',
            client_secret=client_secret,
        )
        client.set_client_metadata({
            'redirect_uris': ['https://a.b'],
            'scope': 'profile address',
            'token_endpoint_auth_method': token_endpoint_auth_method,
            'response_types': ['code'],
            'grant_types': ['authorization_code'],
        })
        self.authorize_url = (
            '/oauth/authorize?response_type=code'
            '&client_id=code-client'
        )
        db.session.add(client)
        db.session.commit()

    def test_missing_code_challenge(self):
        self.prepare_data()
        rv = self.client.get(self.authorize_url + '&code_challenge_method=plain')
        self.assertIn(b'Missing', rv.data)

    def test_has_code_challenge(self):
        self.prepare_data()
        rv = self.client.get(self.authorize_url + '&code_challenge=abc')
        self.assertEqual(rv.data, b'ok')

    def test_invalid_code_challenge_method(self):
        self.prepare_data()
        suffix = '&code_challenge=abc&code_challenge_method=invalid'
        rv = self.client.get(self.authorize_url + suffix)
        self.assertIn(b'Unsupported', rv.data)

    def test_supported_code_challenge_method(self):
        self.prepare_data()
        suffix = '&code_challenge=abc&code_challenge_method=plain'
        rv = self.client.get(self.authorize_url + suffix)
        self.assertEqual(rv.data, b'ok')

    def test_trusted_client_without_code_challenge(self):
        self.prepare_data('client_secret_basic')
        rv = self.client.get(self.authorize_url)
        self.assertEqual(rv.data, b'ok')

        rv = self.client.post(self.authorize_url, data={'user_id': '1'})
        self.assertIn('code=', rv.location)

        params = dict(url_decode(urlparse.urlparse(rv.location).query))

        code = params['code']
        headers = self.create_basic_header('code-client', 'code-secret')
        rv = self.client.post('/oauth/token', data={
            'grant_type': 'authorization_code',
            'code': code,
        }, headers=headers)
        resp = json.loads(rv.data)
        self.assertIn('access_token', resp)

    def test_missing_code_verifier(self):
        self.prepare_data()
        url = self.authorize_url + '&code_challenge=foo'
        rv = self.client.post(url, data={'user_id': '1'})
        self.assertIn('code=', rv.location)

        params = dict(url_decode(urlparse.urlparse(rv.location).query))
        code = params['code']
        rv = self.client.post('/oauth/token', data={
            'grant_type': 'authorization_code',
            'code': code,
            'client_id': 'code-client',
        })
        resp = json.loads(rv.data)
        self.assertIn('Missing', resp['error_description'])

    def test_trusted_client_missing_code_verifier(self):
        self.prepare_data('client_secret_basic')
        url = self.authorize_url + '&code_challenge=foo'
        rv = self.client.post(url, data={'user_id': '1'})
        self.assertIn('code=', rv.location)

        params = dict(url_decode(urlparse.urlparse(rv.location).query))
        code = params['code']
        headers = self.create_basic_header('code-client', 'code-secret')
        rv = self.client.post('/oauth/token', data={
            'grant_type': 'authorization_code',
            'code': code,
        }, headers=headers)
        resp = json.loads(rv.data)
        self.assertIn('Missing', resp['error_description'])

    def test_plain_code_challenge_invalid(self):
        self.prepare_data()
        url = self.authorize_url + '&code_challenge=foo'
        rv = self.client.post(url, data={'user_id': '1'})
        self.assertIn('code=', rv.location)

        params = dict(url_decode(urlparse.urlparse(rv.location).query))
        code = params['code']
        rv = self.client.post('/oauth/token', data={
            'grant_type': 'authorization_code',
            'code': code,
            'code_verifier': 'bar',
            'client_id': 'code-client',
        })
        resp = json.loads(rv.data)
        self.assertIn('Invalid', resp['error_description'])

    def test_plain_code_challenge_failed(self):
        self.prepare_data()
        url = self.authorize_url + '&code_challenge=foo'
        rv = self.client.post(url, data={'user_id': '1'})
        self.assertIn('code=', rv.location)

        params = dict(url_decode(urlparse.urlparse(rv.location).query))
        code = params['code']
        rv = self.client.post('/oauth/token', data={
            'grant_type': 'authorization_code',
            'code': code,
            'code_verifier': generate_token(48),
            'client_id': 'code-client',
        })
        resp = json.loads(rv.data)
        self.assertIn('failed', resp['error_description'])

    def test_plain_code_challenge_success(self):
        self.prepare_data()
        code_verifier = generate_token(48)
        url = self.authorize_url + '&code_challenge=' + code_verifier
        rv = self.client.post(url, data={'user_id': '1'})
        self.assertIn('code=', rv.location)

        params = dict(url_decode(urlparse.urlparse(rv.location).query))
        code = params['code']
        rv = self.client.post('/oauth/token', data={
            'grant_type': 'authorization_code',
            'code': code,
            'code_verifier': code_verifier,
            'client_id': 'code-client',
        })
        resp = json.loads(rv.data)
        self.assertIn('access_token', resp)

    def test_s256_code_challenge_success(self):
        self.prepare_data()
        code_verifier = generate_token(48)
        code_challenge = create_s256_code_challenge(code_verifier)
        url = self.authorize_url + '&code_challenge=' + code_challenge
        url += '&code_challenge_method=S256'

        rv = self.client.post(url, data={'user_id': '1'})
        self.assertIn('code=', rv.location)

        params = dict(url_decode(urlparse.urlparse(rv.location).query))
        code = params['code']
        rv = self.client.post('/oauth/token', data={
            'grant_type': 'authorization_code',
            'code': code,
            'code_verifier': code_verifier,
            'client_id': 'code-client',
        })
        resp = json.loads(rv.data)
        self.assertIn('access_token', resp)

    def test_not_implemented_code_challenge_method(self):
        self.prepare_data()
        url = self.authorize_url + '&code_challenge=foo'
        url += '&code_challenge_method=S128'

        rv = self.client.post(url, data={'user_id': '1'})
        self.assertIn('code=', rv.location)

        params = dict(url_decode(urlparse.urlparse(rv.location).query))
        code = params['code']
        self.assertRaises(
            RuntimeError, self.client.post, '/oauth/token',
            data={
                'grant_type': 'authorization_code',
                'code': code,
                'code_verifier': generate_token(48),
                'client_id': 'code-client',
            }
        )



import unittest
from authlib.oidc.discovery import get_well_known_url, OpenIDProviderMetadata

WELL_KNOWN_URL = '/.well-known/openid-configuration'


class WellKnownTest(unittest.TestCase):
    def test_no_suffix_issuer(self):
        self.assertEqual(
            get_well_known_url('https://authlib.org'),
            WELL_KNOWN_URL
        )
        self.assertEqual(
            get_well_known_url('https://authlib.org/'),
            WELL_KNOWN_URL
        )

    def test_with_suffix_issuer(self):
        self.assertEqual(
            get_well_known_url('https://authlib.org/issuer1'),
            '/issuer1' + WELL_KNOWN_URL
        )
        self.assertEqual(
            get_well_known_url('https://authlib.org/a/b/c'),
            '/a/b/c' + WELL_KNOWN_URL
        )

    def test_with_external(self):
        self.assertEqual(
            get_well_known_url('https://authlib.org', external=True),
            'https://authlib.org' + WELL_KNOWN_URL
        )


class OpenIDProviderMetadataTest(unittest.TestCase):
    def test_validate_jwks_uri(self):
        # required
        metadata = OpenIDProviderMetadata()
        with self.assertRaises(ValueError) as cm:
            metadata.validate_jwks_uri()
            self.assertEqual('"jwks_uri" is required', str(cm.exception))

        metadata = OpenIDProviderMetadata({
            'jwks_uri': 'http://authlib.org/jwks.json'
        })
        with self.assertRaises(ValueError) as cm:
            metadata.validate_jwks_uri()
            self.assertIn('https', str(cm.exception))

        metadata = OpenIDProviderMetadata({
            'jwks_uri': 'https://authlib.org/jwks.json'
        })
        metadata.validate_jwks_uri()

    def test_validate_acr_values_supported(self):
        self._call_validate_array(
            'acr_values_supported',
            ['urn:mace:incommon:iap:silver']
        )

    def test_validate_subject_types_supported(self):
        self._call_validate_array(
            'subject_types_supported',
            ['pairwise', 'public'],
            required=True
        )
        self._call_contains_invalid_value(
            'subject_types_supported',
            ['invalid']
        )

    def test_validate_id_token_signing_alg_values_supported(self):
        self._call_validate_array(
            'id_token_signing_alg_values_supported',
            ['RS256'], required=True,
        )
        metadata = OpenIDProviderMetadata({
            'id_token_signing_alg_values_supported': ['none']
        })
        with self.assertRaises(ValueError) as cm:
            metadata.validate_id_token_signing_alg_values_supported()
            self.assertIn('RS256', str(cm.exception))

    def test_validate_id_token_encryption_alg_values_supported(self):
        self._call_validate_array(
            'id_token_encryption_alg_values_supported',
            ['A128KW']
        )

    def test_validate_id_token_encryption_enc_values_supported(self):
        self._call_validate_array(
            'id_token_encryption_enc_values_supported',
            ['A128GCM']
        )

    def test_validate_userinfo_signing_alg_values_supported(self):
        self._call_validate_array(
            'userinfo_signing_alg_values_supported',
            ['RS256']
        )

    def test_validate_userinfo_encryption_alg_values_supported(self):
        self._call_validate_array(
            'userinfo_encryption_alg_values_supported',
            ['A128KW']
        )

    def test_validate_userinfo_encryption_enc_values_supported(self):
        self._call_validate_array(
            'userinfo_encryption_enc_values_supported',
            ['A128GCM']
        )

    def test_validate_request_object_signing_alg_values_supported(self):
        self._call_validate_array(
            'request_object_signing_alg_values_supported',
            ['none', 'RS256']
        )
        metadata = OpenIDProviderMetadata({
            'request_object_signing_alg_values_supported': ['RS512']
        })
        with self.assertRaises(ValueError) as cm:
            metadata.validate_request_object_signing_alg_values_supported()
            self.assertIn('SHOULD support none and RS256', str(cm.exception))

    def test_validate_request_object_encryption_alg_values_supported(self):
        self._call_validate_array(
            'request_object_encryption_alg_values_supported',
            ['A128KW']
        )

    def test_validate_request_object_encryption_enc_values_supported(self):
        self._call_validate_array(
            'request_object_encryption_enc_values_supported',
            ['A128GCM']
        )

    def test_validate_display_values_supported(self):
        self._call_validate_array(
            'display_values_supported',
            ['page', 'touch']
        )
        self._call_contains_invalid_value(
            'display_values_supported',
            ['invalid']
        )

    def test_validate_claim_types_supported(self):
        self._call_validate_array(
            'claim_types_supported',
            ['normal']
        )
        self._call_contains_invalid_value(
            'claim_types_supported',
            ['invalid']
        )
        metadata = OpenIDProviderMetadata()
        self.assertEqual(metadata.claim_types_supported, ['normal'])

    def test_validate_claims_supported(self):
        self._call_validate_array(
            'claims_supported',
            ['sub']
        )

    def test_validate_claims_locales_supported(self):
        self._call_validate_array(
            'claims_locales_supported',
            ['en-US']
        )

    def test_validate_claims_parameter_supported(self):
        self._call_validate_boolean('claims_parameter_supported')

    def test_validate_request_parameter_supported(self):
        self._call_validate_boolean('request_parameter_supported')

    def test_validate_request_uri_parameter_supported(self):
        self._call_validate_boolean('request_uri_parameter_supported', True)

    def test_validate_require_request_uri_registration(self):
        self._call_validate_boolean('require_request_uri_registration')

    def _call_validate_boolean(self, key, default_value=False):
        def _validate(metadata):
            getattr(metadata, 'validate_' + key)()

        metadata = OpenIDProviderMetadata()
        _validate(metadata)
        self.assertEqual(getattr(metadata, key), default_value)

        metadata = OpenIDProviderMetadata({key: 'str'})
        with self.assertRaises(ValueError) as cm:
            _validate(metadata)
            self.assertIn('MUST be boolean', str(cm.exception))
        metadata = OpenIDProviderMetadata({key: True})
        _validate(metadata)

    def _call_validate_array(self, key, valid_value, required=False):
        def _validate(metadata):
            getattr(metadata, 'validate_' + key)()

        metadata = OpenIDProviderMetadata()
        if required:
            with self.assertRaises(ValueError) as cm:
                _validate(metadata)
                self.assertEqual('"{}" is required'.format(key), str(cm.exception))
        else:
            _validate(metadata)

        # not array
        metadata = OpenIDProviderMetadata({key: 'foo'})
        with self.assertRaises(ValueError) as cm:
            _validate(metadata)
            self.assertIn('JSON array', str(cm.exception))

        # valid
        metadata = OpenIDProviderMetadata({key: valid_value})
        _validate(metadata)

    def _call_contains_invalid_value(self, key, invalid_value):
        metadata = OpenIDProviderMetadata({key: invalid_value})
        with self.assertRaises(ValueError) as cm:
            getattr(metadata, 'validate_' + key)()
            self.assertEqual(
                '"{}" contains invalid values'.format(key),
                str(cm.exception)
            )





from flask import json
from authlib.integrations.sqla_oauth2 import create_revocation_endpoint
from .models import db, User, Client, Token
from .oauth2_server import TestCase
from .oauth2_server import create_authorization_server


RevocationEndpoint = create_revocation_endpoint(db.session, Token)


class RevokeTokenTest(TestCase):
    def prepare_data(self):
        app = self.app
        server = create_authorization_server(app)
        server.register_endpoint(RevocationEndpoint)

        @app.route('/oauth/revoke', methods=['POST'])
        def revoke_token():
            return server.create_endpoint_response('revocation')

        user = User(username='foo')
        db.session.add(user)
        db.session.commit()
        client = Client(
            user_id=user.id,
            client_id='revoke-client',
            client_secret='revoke-secret',
        )
        client.set_client_metadata({
            'scope': 'profile',
            'redirect_uris': ['http://localhost/authorized'],
        })
        db.session.add(client)
        db.session.commit()

    def create_token(self):
        token = Token(
            user_id=1,
            client_id='revoke-client',
            token_type='bearer',
            access_token='a1',
            refresh_token='r1',
            scope='profile',
            expires_in=3600,
        )
        db.session.add(token)
        db.session.commit()

    def test_invalid_client(self):
        self.prepare_data()
        rv = self.client.post('/oauth/revoke')
        resp = json.loads(rv.data)
        self.assertEqual(resp['error'], 'invalid_client')

        headers = {'Authorization': 'invalid token_string'}
        rv = self.client.post('/oauth/revoke', headers=headers)
        resp = json.loads(rv.data)
        self.assertEqual(resp['error'], 'invalid_client')

        headers = self.create_basic_header(
            'invalid-client', 'revoke-secret'
        )
        rv = self.client.post('/oauth/revoke', headers=headers)
        resp = json.loads(rv.data)
        self.assertEqual(resp['error'], 'invalid_client')

        headers = self.create_basic_header(
            'revoke-client', 'invalid-secret'
        )
        rv = self.client.post('/oauth/revoke', headers=headers)
        resp = json.loads(rv.data)
        self.assertEqual(resp['error'], 'invalid_client')

    def test_invalid_token(self):
        self.prepare_data()
        headers = self.create_basic_header(
            'revoke-client', 'revoke-secret'
        )
        rv = self.client.post('/oauth/revoke', headers=headers)
        resp = json.loads(rv.data)
        self.assertEqual(resp['error'], 'invalid_request')

        rv = self.client.post('/oauth/revoke', data={
            'token': 'invalid-token',
        }, headers=headers)
        self.assertEqual(rv.status_code, 200)

        rv = self.client.post('/oauth/revoke', data={
            'token': 'a1',
            'token_type_hint': 'unsupported_token_type',
        }, headers=headers)
        resp = json.loads(rv.data)
        self.assertEqual(resp['error'], 'unsupported_token_type')

        rv = self.client.post('/oauth/revoke', data={
            'token': 'a1',
            'token_type_hint': 'refresh_token',
        }, headers=headers)
        self.assertEqual(rv.status_code, 200)

    def test_revoke_token_with_hint(self):
        self.prepare_data()
        self.create_token()
        headers = self.create_basic_header(
            'revoke-client', 'revoke-secret'
        )
        rv = self.client.post('/oauth/revoke', data={
            'token': 'a1',
            'token_type_hint': 'access_token',
        }, headers=headers)
        self.assertEqual(rv.status_code, 200)

    def test_revoke_token_without_hint(self):
        self.prepare_data()
        self.create_token()
        headers = self.create_basic_header(
            'revoke-client', 'revoke-secret'
        )
        rv = self.client.post('/oauth/revoke', data={
            'token': 'a1',
        }, headers=headers)
        self.assertEqual(rv.status_code, 200)



from tests.util import decode_response
from .oauth1_server import db, User, Client
from .oauth1_server import (
    TestCase,
    create_authorization_server,
)


class AuthorizationWithCacheTest(TestCase):
    USE_CACHE = True

    def prepare_data(self):
        create_authorization_server(self.app, self.USE_CACHE, self.USE_CACHE)
        user = User(username='foo')
        db.session.add(user)
        db.session.commit()
        client = Client(
            user_id=user.id,
            client_id='client',
            client_secret='secret',
            default_redirect_uri='https://a.b',
        )
        db.session.add(client)
        db.session.commit()

    def test_invalid_authorization(self):
        self.prepare_data()
        url = '/oauth/authorize'

        # case 1
        rv = self.client.post(url, data={'user_id': '1'})
        data = decode_response(rv.data)
        self.assertEqual(data['error'], 'missing_required_parameter')
        self.assertIn('oauth_token', data['error_description'])

        # case 2
        rv = self.client.post(url, data={'user_id': '1', 'oauth_token': 'a'})
        data = decode_response(rv.data)
        self.assertEqual(data['error'], 'invalid_token')

    def test_authorize_denied(self):
        self.prepare_data()
        initiate_url = '/oauth/initiate'
        authorize_url = '/oauth/authorize'

        rv = self.client.post(initiate_url, data={
            'oauth_consumer_key': 'client',
            'oauth_callback': 'oob',
            'oauth_signature_method': 'PLAINTEXT',
            'oauth_signature': 'secret&'
        })
        data = decode_response(rv.data)
        self.assertIn('oauth_token', data)

        rv = self.client.post(authorize_url, data={
            'oauth_token': data['oauth_token']
        })
        self.assertEqual(rv.status_code, 302)
        self.assertIn('access_denied', rv.headers['Location'])
        self.assertIn('https://a.b', rv.headers['Location'])

        rv = self.client.post(initiate_url, data={
            'oauth_consumer_key': 'client',
            'oauth_callback': 'https://i.test',
            'oauth_signature_method': 'PLAINTEXT',
            'oauth_signature': 'secret&'
        })
        data = decode_response(rv.data)
        self.assertIn('oauth_token', data)

        rv = self.client.post(authorize_url, data={
            'oauth_token': data['oauth_token']
        })
        self.assertEqual(rv.status_code, 302)
        self.assertIn('access_denied', rv.headers['Location'])
        self.assertIn('https://i.test', rv.headers['Location'])

    def test_authorize_granted(self):
        self.prepare_data()
        initiate_url = '/oauth/initiate'
        authorize_url = '/oauth/authorize'

        rv = self.client.post(initiate_url, data={
            'oauth_consumer_key': 'client',
            'oauth_callback': 'oob',
            'oauth_signature_method': 'PLAINTEXT',
            'oauth_signature': 'secret&'
        })
        data = decode_response(rv.data)
        self.assertIn('oauth_token', data)

        rv = self.client.post(authorize_url, data={
            'user_id': '1',
            'oauth_token': data['oauth_token']
        })
        self.assertEqual(rv.status_code, 302)
        self.assertIn('oauth_verifier', rv.headers['Location'])
        self.assertIn('https://a.b', rv.headers['Location'])

        rv = self.client.post(initiate_url, data={
            'oauth_consumer_key': 'client',
            'oauth_callback': 'https://i.test',
            'oauth_signature_method': 'PLAINTEXT',
            'oauth_signature': 'secret&'
        })
        data = decode_response(rv.data)
        self.assertIn('oauth_token', data)

        rv = self.client.post(authorize_url, data={
            'user_id': '1',
            'oauth_token': data['oauth_token']
        })
        self.assertEqual(rv.status_code, 302)
        self.assertIn('oauth_verifier', rv.headers['Location'])
        self.assertIn('https://i.test', rv.headers['Location'])


class AuthorizationNoCacheTest(AuthorizationWithCacheTest):
    USE_CACHE = False



from __future__ import unicode_literals, print_function
import mock
import requests
from unittest import TestCase
from io import StringIO

from authlib.oauth1 import (
    SIGNATURE_PLAINTEXT,
    SIGNATURE_RSA_SHA1,
    SIGNATURE_TYPE_BODY,
    SIGNATURE_TYPE_QUERY,
)
from authlib.oauth1.rfc5849.util import escape
from authlib.common.encoding import to_unicode, unicode_type
from authlib.integrations.requests_client import OAuth1Session, OAuthError
from tests.client_base import mock_text_response
from tests.util import read_file_path


TEST_RSA_OAUTH_SIGNATURE = (
    "j8WF8PGjojT82aUDd2EL%2Bz7HCoHInFzWUpiEKMCy%2BJ2cYHWcBS7mXlmFDLgAKV0"
    "P%2FyX4TrpXODYnJ6dRWdfghqwDpi%2FlQmB2jxCiGMdJoYxh3c5zDf26gEbGdP6D7O"
    "Ssp5HUnzH6sNkmVjuE%2FxoJcHJdc23H6GhOs7VJ2LWNdbhKWP%2FMMlTrcoQDn8lz"
    "%2Fb24WsJ6ae1txkUzpFOOlLM8aTdNtGL4OtsubOlRhNqnAFq93FyhXg0KjzUyIZzmMX"
    "9Vx90jTks5QeBGYcLE0Op2iHb2u%2FO%2BEgdwFchgEwE5LgMUyHUI4F3Wglp28yHOAM"
    "jPkI%2FkWMvpxtMrU3Z3KN31WQ%3D%3D"
)


class OAuth1SessionTest(TestCase):

    def test_no_client_id(self):
        self.assertRaises(ValueError, lambda: OAuth1Session(None))

    def test_signature_types(self):
        def verify_signature(getter):
            def fake_send(r, **kwargs):
                signature = to_unicode(getter(r))
                self.assertIn('oauth_signature', signature)
                resp = mock.MagicMock(spec=requests.Response)
                resp.cookies = []
                return resp
            return fake_send

        header = OAuth1Session('foo')
        header.send = verify_signature(lambda r: r.headers['Authorization'])
        header.post('https://i.b')

        query = OAuth1Session('foo', signature_type=SIGNATURE_TYPE_QUERY)
        query.send = verify_signature(lambda r: r.url)
        query.post('https://i.b')

        body = OAuth1Session('foo', signature_type=SIGNATURE_TYPE_BODY)
        headers = {'Content-Type': 'application/x-www-form-urlencoded'}
        body.send = verify_signature(lambda r: r.body)
        body.post('https://i.b', headers=headers, data='')

    @mock.patch('authlib.oauth1.rfc5849.client_auth.generate_timestamp')
    @mock.patch('authlib.oauth1.rfc5849.client_auth.generate_nonce')
    def test_signature_methods(self, generate_nonce, generate_timestamp):
        generate_nonce.return_value = 'abc'
        generate_timestamp.return_value = '123'

        signature = ', '.join([
            'OAuth oauth_nonce="abc"',
            'oauth_timestamp="123"',
            'oauth_version="1.0"',
            'oauth_signature_method="HMAC-SHA1"',
            'oauth_consumer_key="foo"',
            'oauth_signature="h2sRqLArjhlc5p3FTkuNogVHlKE%3D"'
        ])
        auth = OAuth1Session('foo')
        auth.send = self.verify_signature(signature)
        auth.post('https://i.b')

        signature = (
            'OAuth '
            'oauth_nonce="abc", oauth_timestamp="123", oauth_version="1.0", '
            'oauth_signature_method="PLAINTEXT", oauth_consumer_key="foo", '
            'oauth_signature="%26"'
        )
        auth = OAuth1Session('foo', signature_method=SIGNATURE_PLAINTEXT)
        auth.send = self.verify_signature(signature)
        auth.post('https://i.b')

        signature = (
            'OAuth '
            'oauth_nonce="abc", oauth_timestamp="123", oauth_version="1.0", '
            'oauth_signature_method="RSA-SHA1", oauth_consumer_key="foo", '
            'oauth_signature="{sig}"'
        ).format(sig=TEST_RSA_OAUTH_SIGNATURE)

        rsa_key = read_file_path('rsa_private.pem')
        auth = OAuth1Session(
            'foo', signature_method=SIGNATURE_RSA_SHA1, rsa_key=rsa_key)
        auth.send = self.verify_signature(signature)
        auth.post('https://i.b')

    @mock.patch('authlib.oauth1.rfc5849.client_auth.generate_timestamp')
    @mock.patch('authlib.oauth1.rfc5849.client_auth.generate_nonce')
    def test_binary_upload(self, generate_nonce, generate_timestamp):
        generate_nonce.return_value = 'abc'
        generate_timestamp.return_value = '123'
        fake_xml = StringIO('hello world')
        headers = {'Content-Type': 'application/xml'}
        signature = (
            'OAuth oauth_nonce="abc", oauth_timestamp="123", oauth_version="1.0", '
            'oauth_signature_method="HMAC-SHA1", oauth_consumer_key="foo", '
            'oauth_signature="h2sRqLArjhlc5p3FTkuNogVHlKE%3D"'
        )
        auth = OAuth1Session('foo')
        auth.send = self.verify_signature(signature)
        auth.post('https://i.b', headers=headers, files=[('fake', fake_xml)])

    @mock.patch('authlib.oauth1.rfc5849.client_auth.generate_timestamp')
    @mock.patch('authlib.oauth1.rfc5849.client_auth.generate_nonce')
    def test_nonascii(self, generate_nonce, generate_timestamp):
        generate_nonce.return_value = 'abc'
        generate_timestamp.return_value = '123'
        signature = (
            'OAuth oauth_nonce="abc", oauth_timestamp="123", oauth_version="1.0", '
            'oauth_signature_method="HMAC-SHA1", oauth_consumer_key="foo", '
            'oauth_signature="W0haoue5IZAZoaJiYCtfqwMf8x8%3D"'
        )
        auth = OAuth1Session('foo')
        auth.send = self.verify_signature(signature)
        auth.post('https://i.b?cjk=%E5%95%A6%E5%95%A6')

    def test_redirect_uri(self):
        sess = OAuth1Session('foo')
        self.assertIsNone(sess.redirect_uri)
        url = 'https://i.b'
        sess.redirect_uri = url
        self.assertEqual(sess.redirect_uri, url)

    def test_set_token(self):
        sess = OAuth1Session('foo')
        try:
            sess.token = {}
        except OAuthError as exc:
            self.assertEqual(exc.error, 'missing_token')

        sess.token = {'oauth_token': 'a', 'oauth_token_secret': 'b'}
        self.assertIsNone(sess.token['oauth_verifier'])
        sess.token = {'oauth_token': 'a', 'oauth_verifier': 'c'}
        self.assertEqual(sess.token['oauth_token_secret'], 'b')
        self.assertEqual(sess.token['oauth_verifier'], 'c')

        sess.token = None
        self.assertIsNone(sess.token['oauth_token'])
        self.assertIsNone(sess.token['oauth_token_secret'])
        self.assertIsNone(sess.token['oauth_verifier'])

    def test_create_authorization_url(self):
        auth = OAuth1Session('foo')
        url = 'https://example.comm/authorize'
        token = 'asluif023sf'
        auth_url = auth.create_authorization_url(url, request_token=token)
        self.assertEqual(auth_url, url + '?oauth_token=' + token)
        redirect_uri = 'https://c.b'
        auth = OAuth1Session('foo', redirect_uri=redirect_uri)
        auth_url = auth.create_authorization_url(url, request_token=token)
        self.assertIn(escape(redirect_uri), auth_url)

    def test_parse_response_url(self):
        url = 'https://i.b/callback?oauth_token=foo&oauth_verifier=bar'
        auth = OAuth1Session('foo')
        resp = auth.parse_authorization_response(url)
        self.assertEqual(resp['oauth_token'], 'foo')
        self.assertEqual(resp['oauth_verifier'], 'bar')
        for k, v in resp.items():
            self.assertTrue(isinstance(k, unicode_type))
            self.assertTrue(isinstance(v, unicode_type))

    def test_fetch_request_token(self):
        auth = OAuth1Session('foo')
        auth.send = mock_text_response('oauth_token=foo')
        resp = auth.fetch_request_token('https://example.com/token')
        self.assertEqual(resp['oauth_token'], 'foo')
        for k, v in resp.items():
            self.assertTrue(isinstance(k, unicode_type))
            self.assertTrue(isinstance(v, unicode_type))

        resp = auth.fetch_request_token('https://example.com/token', realm='A')
        self.assertEqual(resp['oauth_token'], 'foo')
        resp = auth.fetch_request_token('https://example.com/token', realm=['A', 'B'])
        self.assertEqual(resp['oauth_token'], 'foo')

    def test_fetch_request_token_with_optional_arguments(self):
        auth = OAuth1Session('foo')
        auth.send = mock_text_response('oauth_token=foo')
        resp = auth.fetch_request_token('https://example.com/token',
                                        verify=False, stream=True)
        self.assertEqual(resp['oauth_token'], 'foo')
        for k, v in resp.items():
            self.assertTrue(isinstance(k, unicode_type))
            self.assertTrue(isinstance(v, unicode_type))

    def test_fetch_access_token(self):
        auth = OAuth1Session('foo', verifier='bar')
        auth.send = mock_text_response('oauth_token=foo')
        resp = auth.fetch_access_token('https://example.com/token')
        self.assertEqual(resp['oauth_token'], 'foo')
        for k, v in resp.items():
            self.assertTrue(isinstance(k, unicode_type))
            self.assertTrue(isinstance(v, unicode_type))

        auth = OAuth1Session('foo', verifier='bar')
        auth.send = mock_text_response('{"oauth_token":"foo"}')
        resp = auth.fetch_access_token('https://example.com/token')
        self.assertEqual(resp['oauth_token'], 'foo')

        auth = OAuth1Session('foo')
        auth.send = mock_text_response('oauth_token=foo')
        resp = auth.fetch_access_token(
            'https://example.com/token', verifier='bar')
        self.assertEqual(resp['oauth_token'], 'foo')

    def test_fetch_access_token_with_optional_arguments(self):
        auth = OAuth1Session('foo', verifier='bar')
        auth.send = mock_text_response('oauth_token=foo')
        resp = auth.fetch_access_token('https://example.com/token',
                                       verify=False, stream=True)
        self.assertEqual(resp['oauth_token'], 'foo')
        for k, v in resp.items():
            self.assertTrue(isinstance(k, unicode_type))
            self.assertTrue(isinstance(v, unicode_type))

    def _test_fetch_access_token_raises_error(self, session):
        """Assert that an error is being raised whenever there's no verifier
        passed in to the client.
        """
        session.send = mock_text_response('oauth_token=foo')

        # Use a try-except block so that we can assert on the exception message
        # being raised and also keep the Python2.6 compatibility where
        # assertRaises is not a context manager.
        try:
            session.fetch_access_token('https://example.com/token')
        except OAuthError as exc:
            self.assertEqual(exc.error, 'missing_verifier')

    def test_fetch_token_invalid_response(self):
        auth = OAuth1Session('foo')
        auth.send = mock_text_response('not valid urlencoded response!')
        self.assertRaises(
            ValueError, auth.fetch_request_token, 'https://example.com/token')

        for code in (400, 401, 403):
            auth.send = mock_text_response('valid=response', code)
            # use try/catch rather than self.assertRaises, so we can
            # assert on the properties of the exception
            try:
                auth.fetch_request_token('https://example.com/token')
            except OAuthError as err:
                self.assertEqual(err.error, 'fetch_token_denied')
            else:  # no exception raised
                self.fail("ValueError not raised")

    def test_fetch_access_token_missing_verifier(self):
        self._test_fetch_access_token_raises_error(OAuth1Session('foo'))

    def test_fetch_access_token_has_verifier_is_none(self):
        session = OAuth1Session('foo')
        session.auth.verifier = None
        self._test_fetch_access_token_raises_error(session)

    def verify_signature(self, signature):
        def fake_send(r, **kwargs):
            auth_header = to_unicode(r.headers['Authorization'])
            self.assertEqual(auth_header, signature)
            resp = mock.MagicMock(spec=requests.Response)
            resp.cookies = []
            return resp
        return fake_send



import unittest
import base64
from authlib.oauth2.rfc6749 import parameters
from authlib.oauth2.rfc6749 import util
from authlib.oauth2.rfc6749 import errors


class OAuth2ParametersTest(unittest.TestCase):
    def test_parse_authorization_code_response(self):
        self.assertRaises(
            errors.MissingCodeException,
            parameters.parse_authorization_code_response,
            'https://i.b/?state=c'
        )

        self.assertRaises(
            errors.MismatchingStateException,
            parameters.parse_authorization_code_response,
            'https://i.b/?code=a&state=c',
            'b'
        )

        url = 'https://i.b/?code=a&state=c'
        rv = parameters.parse_authorization_code_response(url, 'c')
        self.assertEqual(rv, {'code': 'a', 'state': 'c'})

    def test_parse_implicit_response(self):
        self.assertRaises(
            errors.MissingTokenException,
            parameters.parse_implicit_response,
            'https://i.b/#a=b'
        )

        self.assertRaises(
            errors.MissingTokenTypeException,
            parameters.parse_implicit_response,
            'https://i.b/#access_token=a'
        )

        self.assertRaises(
            errors.MismatchingStateException,
            parameters.parse_implicit_response,
            'https://i.b/#access_token=a&token_type=bearer&state=c',
            'abc'
        )

        url = 'https://i.b/#access_token=a&token_type=bearer&state=c'
        rv = parameters.parse_implicit_response(url, 'c')
        self.assertEqual(
            rv,
            {'access_token': 'a', 'token_type': 'bearer', 'state': 'c'}
        )


class OAuth2UtilTest(unittest.TestCase):
    def test_list_to_scope(self):
        self.assertEqual(util.list_to_scope(['a', 'b']), 'a b')
        self.assertEqual(util.list_to_scope('a b'), 'a b')
        self.assertIsNone(util.list_to_scope(None))

    def test_scope_to_list(self):
        self.assertEqual(util.scope_to_list('a b'), ['a', 'b'])
        self.assertEqual(util.scope_to_list(['a', 'b']), ['a', 'b'])
        self.assertIsNone(util.scope_to_list(None))

    def test_extract_basic_authorization(self):
        self.assertEqual(util.extract_basic_authorization({}), (None, None))
        self.assertEqual(
            util.extract_basic_authorization({'Authorization': 'invalid'}),
            (None, None)
        )

        text = 'Basic invalid-base64'
        self.assertEqual(
            util.extract_basic_authorization({'Authorization': text}),
            (None, None)
        )

        text = 'Basic {}'.format(base64.b64encode(b'a').decode())
        self.assertEqual(
            util.extract_basic_authorization({'Authorization': text}),
            ('a', None)
        )

        text = 'Basic {}'.format(base64.b64encode(b'a:b').decode())
        self.assertEqual(
            util.extract_basic_authorization({'Authorization': text}),
            ('a', 'b')
        )



import time
import pytest
from authlib.integrations.httpx_client import AssertionClient
from tests.py3.utils import MockDispatch


default_token = {
    'token_type': 'Bearer',
    'access_token': 'a',
    'refresh_token': 'b',
    'expires_in': '3600',
    'expires_at': int(time.time()) + 3600,
}


@pytest.mark.asyncio
def test_refresh_token():
    def verifier(request):
        content = request.form
        if str(request.url) == 'https://i.b/token':
            assert 'assertion' in content

    with AssertionClient(
        'https://i.b/token',
        grant_type=AssertionClient.JWT_BEARER_GRANT_TYPE,
        issuer='foo',
        subject='foo',
        audience='foo',
        alg='HS256',
        key='secret',
        app=MockDispatch(default_token, assert_func=verifier)
    ) as client:
        client.get('https://i.b')

    # trigger more case
    now = int(time.time())
    with AssertionClient(
        'https://i.b/token',
        issuer='foo',
        subject=None,
        audience='foo',
        issued_at=now,
        expires_at=now + 3600,
        header={'alg': 'HS256'},
        key='secret',
        scope='email',
        claims={'test_mode': 'true'},
        app=MockDispatch(default_token, assert_func=verifier)
    ) as client:
        client.get('https://i.b')
        client.get('https://i.b')


@pytest.mark.asyncio
def test_without_alg():
    with AssertionClient(
        'https://i.b/token',
        issuer='foo',
        subject='foo',
        audience='foo',
        key='secret',
        app=MockDispatch(default_token)
    ) as client:
        with pytest.raises(ValueError):
            client.get('https://i.b')



import json
from authlib.integrations.django_oauth2 import ResourceProtector, BearerTokenValidator
from django.http import JsonResponse
from .models import User, Client, OAuth2Token
from .oauth2_server import TestCase


require_oauth = ResourceProtector()
require_oauth.register_token_validator(BearerTokenValidator(OAuth2Token))


class ResourceProtectorTest(TestCase):
    def prepare_data(self, expires_in=3600, scope='profile'):
        user = User(username='foo')
        user.save()
        client = Client(
            user_id=user.pk,
            client_id='client',
            client_secret='secret',
            scope='profile',
        )
        client.save()

        token = OAuth2Token(
            user_id=user.pk,
            client_id=client.client_id,
            token_type='bearer',
            access_token='a1',
            scope=scope,
            expires_in=expires_in,
        )
        token.save()

    def test_invalid_token(self):
        @require_oauth('profile')
        def get_user_profile(request):
            user = request.oauth_token.user
            return JsonResponse(dict(sub=user.pk, username=user.username))

        self.prepare_data()

        request = self.factory.get('/user')
        resp = get_user_profile(request)
        self.assertEqual(resp.status_code, 401)
        data = json.loads(resp.content)
        self.assertEqual(data['error'], 'missing_authorization')

        request = self.factory.get('/user', HTTP_AUTHORIZATION='invalid token')
        resp = get_user_profile(request)
        self.assertEqual(resp.status_code, 401)
        data = json.loads(resp.content)
        self.assertEqual(data['error'], 'unsupported_token_type')

        request = self.factory.get('/user', HTTP_AUTHORIZATION='bearer token')
        resp = get_user_profile(request)
        self.assertEqual(resp.status_code, 401)
        data = json.loads(resp.content)
        self.assertEqual(data['error'], 'invalid_token')

    def test_expired_token(self):
        self.prepare_data(0)

        @require_oauth('profile')
        def get_user_profile(request):
            user = request.oauth_token.user
            return JsonResponse(dict(sub=user.pk, username=user.username))

        request = self.factory.get('/user', HTTP_AUTHORIZATION='bearer a1')
        resp = get_user_profile(request)
        self.assertEqual(resp.status_code, 401)
        data = json.loads(resp.content)
        self.assertEqual(data['error'], 'invalid_token')

    def test_insufficient_token(self):
        self.prepare_data()

        @require_oauth('email')
        def get_user_email(request):
            user = request.oauth_token.user
            return JsonResponse(dict(email=user.email))

        request = self.factory.get('/user/email', HTTP_AUTHORIZATION='bearer a1')
        resp = get_user_email(request)
        self.assertEqual(resp.status_code, 403)
        data = json.loads(resp.content)
        self.assertEqual(data['error'], 'insufficient_scope')

    def test_access_resource(self):
        self.prepare_data()

        @require_oauth('profile', optional=True)
        def get_user_profile(request):
            if request.oauth_token:
                user = request.oauth_token.user
                return JsonResponse(dict(sub=user.pk, username=user.username))
            return JsonResponse(dict(sub=0, username='anonymous'))

        request = self.factory.get('/user')
        resp = get_user_profile(request)
        self.assertEqual(resp.status_code, 200)
        data = json.loads(resp.content)
        self.assertEqual(data['username'], 'anonymous')

        request = self.factory.get('/user', HTTP_AUTHORIZATION='bearer a1')
        resp = get_user_profile(request)
        self.assertEqual(resp.status_code, 200)
        data = json.loads(resp.content)
        self.assertEqual(data['username'], 'foo')

    def test_scope_operator(self):
        self.prepare_data()

        @require_oauth('profile email', 'AND')
        def operator_and(request):
            user = request.oauth_token.user
            return JsonResponse(dict(sub=user.pk, username=user.username))

        @require_oauth('profile email', 'OR')
        def operator_or(request):
            user = request.oauth_token.user
            return JsonResponse(dict(sub=user.pk, username=user.username))

        request = self.factory.get('/user', HTTP_AUTHORIZATION='bearer a1')
        resp = operator_and(request)
        self.assertEqual(resp.status_code, 403)
        data = json.loads(resp.content)
        self.assertEqual(data['error'], 'insufficient_scope')

        resp = operator_or(request)
        self.assertEqual(resp.status_code, 200)
        data = json.loads(resp.content)
        self.assertEqual(data['username'], 'foo')

        def scope_operator(token_scopes, resource_scopes):
            return 'profile' in token_scopes and 'email' not in token_scopes

        @require_oauth(operator=scope_operator)
        def operator_func(request):
            user = request.oauth_token.user
            return JsonResponse(dict(sub=user.pk, username=user.username))

        resp = operator_func(request)
        self.assertEqual(resp.status_code, 200)
        data = json.loads(resp.content)
        self.assertEqual(data['username'], 'foo')



import unittest
from authlib.jose.errors import MissingClaimError, InvalidClaimError
from authlib.oidc.core import CodeIDToken, ImplicitIDToken, HybridIDToken
from authlib.oidc.core import UserInfo, get_claim_cls_by_response_type


class IDTokenTest(unittest.TestCase):
    def test_essential_claims(self):
        claims = CodeIDToken({}, {})
        self.assertRaises(MissingClaimError, claims.validate)
        claims = CodeIDToken({
            'iss': '1',
            'sub': '1',
            'aud': '1',
            'exp': 10000,
            'iat': 100
        }, {})
        claims.validate(1000)

    def test_validate_auth_time(self):
        claims = CodeIDToken({
            'iss': '1',
            'sub': '1',
            'aud': '1',
            'exp': 10000,
            'iat': 100
        }, {})
        claims.params = {'max_age': 100}
        self.assertRaises(MissingClaimError, claims.validate, 1000)

        claims['auth_time'] = 'foo'
        self.assertRaises(InvalidClaimError, claims.validate, 1000)

    def test_validate_nonce(self):
        claims = CodeIDToken({
            'iss': '1',
            'sub': '1',
            'aud': '1',
            'exp': 10000,
            'iat': 100
        }, {})
        claims.params = {'nonce': 'foo'}
        self.assertRaises(MissingClaimError, claims.validate, 1000)
        claims['nonce'] = 'bar'
        self.assertRaises(InvalidClaimError, claims.validate, 1000)
        claims['nonce'] = 'foo'
        claims.validate(1000)

    def test_validate_amr(self):
        claims = CodeIDToken({
            'iss': '1',
            'sub': '1',
            'aud': '1',
            'exp': 10000,
            'iat': 100,
            'amr': 'invalid'
        }, {})
        self.assertRaises(InvalidClaimError, claims.validate, 1000)

    def test_validate_azp(self):
        claims = CodeIDToken({
            'iss': '1',
            'sub': '1',
            'aud': '1',
            'exp': 10000,
            'iat': 100,
        }, {})
        claims.params = {'client_id': '2'}
        self.assertRaises(MissingClaimError, claims.validate, 1000)

        claims['azp'] = '1'
        self.assertRaises(InvalidClaimError, claims.validate, 1000)

        claims['azp'] = '2'
        claims.validate(1000)

    def test_validate_at_hash(self):
        claims = CodeIDToken({
            'iss': '1',
            'sub': '1',
            'aud': '1',
            'exp': 10000,
            'iat': 100,
            'at_hash': 'a'
        }, {})
        claims.params = {'access_token': 'a'}

        # invalid alg won't raise
        claims.header = {'alg': 'HS222'}
        claims.validate(1000)

        claims.header = {'alg': 'HS256'}
        self.assertRaises(InvalidClaimError, claims.validate, 1000)

    def test_implicit_id_token(self):
        claims = ImplicitIDToken({
            'iss': '1',
            'sub': '1',
            'aud': '1',
            'exp': 10000,
            'iat': 100,
            'nonce': 'a'
        }, {})
        claims.params = {'access_token': 'a'}
        self.assertRaises(MissingClaimError, claims.validate, 1000)

    def test_hybrid_id_token(self):
        claims = HybridIDToken({
            'iss': '1',
            'sub': '1',
            'aud': '1',
            'exp': 10000,
            'iat': 100,
            'nonce': 'a'
        }, {})
        claims.validate(1000)

        claims.params = {'code': 'a'}
        self.assertRaises(MissingClaimError, claims.validate, 1000)

        # invalid alg won't raise
        claims.header = {'alg': 'HS222'}
        claims['c_hash'] = 'a'
        claims.validate(1000)

        claims.header = {'alg': 'HS256'}
        self.assertRaises(InvalidClaimError, claims.validate, 1000)

    def test_get_claim_cls_by_response_type(self):
        cls = get_claim_cls_by_response_type('id_token')
        self.assertEqual(cls, ImplicitIDToken)
        cls = get_claim_cls_by_response_type('code')
        self.assertEqual(cls, CodeIDToken)
        cls = get_claim_cls_by_response_type('code id_token')
        self.assertEqual(cls, HybridIDToken)
        cls = get_claim_cls_by_response_type('none')
        self.assertIsNone(cls)


class UserInfoTest(unittest.TestCase):
    def test_getattribute(self):
        user = UserInfo({'sub': '1'})
        self.assertEqual(user.sub, '1')
        self.assertIsNone(user.email, None)
        self.assertRaises(AttributeError, lambda: user.invalid)
